
///////////////////////////////////////////////
//original zdrojak: Jan Koci (koci.opengl.cz)//
//upravil:          Peter Adamk            // 
///////////////////////////////////////////////


#ifndef _Q3BSP_H_
#define _Q3BSP_H_


#include "q3bitset.h"
#include "q3enums.h"
#include "q3structs.h"



//---------------------------------------------------
//CUSTOM VERTEX QBSP
//---------------------------------------------------
struct CUSTOMVERTEXQBSP
{ 
	D3DXVECTOR3 pos; // pozicia
    DWORD    color;       // farba
    FLOAT       tu, tv;   // texturove suradnice
	FLOAT       tu2, tv2;   // suradnice lightmapy

};
#define D3DFVF_CUSTOMVERTEXQBSP (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX2)


//------------------------------------------------------------------
// Name: QBSP class
// Desc: Trieda Q3 BSP
//------------------------------------------------------------------
class QBSP
{
private:

	LPDIRECT3DVERTEXBUFFER9 g_pVB     ;  //vertex buffer
	LPDIRECT3DTEXTURE9    *Texture    ; //texturovy zasobnik
    LPDIRECT3DTEXTURE9    *LightMap   ; //lightmapy

public:

	//kontruktor detruktor
	QBSP();
	~QBSP();

	 // The number of verts in the model
     // Poet vrchol scny
     int m_iNumOfVerts;

     // The number of faces in the model
     // Poet ploek scny
     int m_iNumOfFaces;

     // The number of texture maps
     // Poet textur
     int m_iNumOfTextures;

     // The number of light maps
     // Poet lightmap textur
     int m_iNumOfLightmaps;

     // The number of nodes
     // Poet uzl
     int m_iNumOfNodes;

     // The number of leafs
     // Poe list
     int m_iNumOfLeafs;

     // The number of leaf faces
     // Poet ploek v listu
     int m_iNumOfLeafFaces;

     // The number of planes
     // Poet dlicch ploch
     int m_iNumOfPlanes;

     // This will store how many faces are drawn and are seen by the camera
     // Kontroln promnn, kter ukld, kolik ploek se renderuje
     int m_iVisibleFaces;

 	// Poet koliznch srek v listu 
	int m_iNumOfLeafBrushes;

	// Poet koliznch srek 
	int m_iNumOfBrushes;

	// Poet koliznch ploch srek 
	int m_iNumOfBrushSides;

	// Kontroln promnn, kter ukld, kolik koliz se provd
	int m_iVisibleBrushes;

	int m_pppiBoundingBox[6][4][3];		// [6]poet stran, [4]poet vrchol strany, [3]xyz pozice vrcholu

	int m_iNumSkyTex;	// Poet textur nebe
	int m_piSkyTex[10];	// Id hodnoty textur nebe

	int m_iNumOfShaders;	// Poet efekt
	int *m_piShaderBrushIndex;	// Index efektu, potebn pro zamezen detekce kolize nap. z mlhou 

	int m_iNumOfModels;	// Poet model
	int *m_piModelBrushIndex;	// Index modelu, potebn pro zamezen detekce kolize 

	int m_iNumOfMeshVertices;	// Poet vrchol me
	int *m_piMeshVertices;		// Pole vrchol me


     // The object's vertices
     // Vrcholy scny
     TBspVertex  *m_ptBspVertex;

     // The faces information of the object
     // Ploky scny
     TBspFace    *m_ptBspFace;

     // The nodes information of the BSP scene
     // Uzly BSP scny
     TBspNode    *m_ptBspNode;

     // The leafs information of the BSP scene
     // Listy BSP scny
     TBspLeaf    *m_ptBspLeaf;

     // The splitter planes of the BSP scene
     // Dlic plochy BSP scny
     TBspPlane   *m_ptBspPlane;

	// The Brushes
	// Kolizn srky
	TBspBrush   *m_ptBspBrush;

	// The splitter planes of the Brush side
	// Dlic plochy koliznch srek
	TBspBrushSides	*m_ptBspBrushSides;

     // The leaf faces of the BSP scene
     // Ploky list BSP scny
     int         *m_piLeafFaces;

     // The clusters of the BSP scene
     // Data pro detekci viditelnosti portl
     TBspVisData  m_tBspVisData;

	// The brushes 
 	// Kolizn srky listu
   	int *m_piLeafBrushes; 

	TBspShader *m_ptBspShader;	// Efekty

	TBspModel *m_ptBspModel;	// Modely

	 
     // The bitset for the faces that have/haven't been drawn
     // Promnn pro detekci, zda m bt ploka vykreslena i ne
     K_CBitset m_BitsetFacesDrawn;

	

	 ////////////////////
	 //POMOCNE FUNKCIE //
	 ////////////////////

	 void FillVertexBuffer();
	 void RenderFace(int i_face_index);
	 int  FindLeaf(VECTOR3D CamPos);
	 bool CheckBrushCollision(int i_leaf_index);
	 int IsClusterVisible(int i_camera_in_cluster, int i_test_cluster);

	 ////////////////////
	 //    FUNKCIE     //
	 ////////////////////
	 bool LoadBSP(const char *pc_filename,float Gamma);
	 void RenderBSP();
	 bool ColliseBSP(VECTOR3D Point);
    
};








#endif //_Q3BSP_H_